import axios from "axios";
import { TIME_SECOND_MS } from "./Consts";
import { getCurrentTimeStamp_MS } from "./Functions";
const tunnel = require('tunnel');

export function getAxios(timeout: number = 10 * TIME_SECOND_MS, useProxy = false) {
    const axiosInstance = axios.create();

    axiosInstance.interceptors.request.use(config => {
        config.timeout = timeout;
        config.cancelToken = new axios.CancelToken(cancel => setTimeout(cancel, timeout));
        config.headers["Request-Timestamp"] = getCurrentTimeStamp_MS().toString();

        return config;
    });

    axiosInstance.interceptors.response.use(response => {
        const startTime = Number(response.config.headers["Request-Timestamp"]);
        const endTime = getCurrentTimeStamp_MS();

        response.headers["Response-Timestamp"] = endTime.toString();
        response.headers["Request-Duration"] = (endTime - startTime).toString();

        return response;
    });

    if (useProxy) {
        const tunnelProxy = tunnel.httpsOverHttp({
            proxy: {
                host: '127.0.0.1',
                port: '1080',
                proxyAuth: 'user:pass',
            },
        });

        axiosInstance.defaults.httpAgent = tunnelProxy;
        axiosInstance.defaults.httpsAgent = tunnelProxy;
    }

    return axiosInstance;
}
